/***************************************************************************
 *   TCP/IP client socket SDL_RWops stream implimentation                  *
 *                                                                         *
 *   Copyright (C) 2004 by Tyler Montbriand                                *
 *   tsm@accesscomm.ca                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <SDL/SDL_net.h>
#include "SDL_rwsock.h"

#define SDL_SOCK_TYPE 0xdead0002

static int rw_sock_seek (SDL_RWops *c, int offset, int whence);
static int rw_sock_read (SDL_RWops *c, void *ptr, int size, int maxnum);
static int rw_sock_write(SDL_RWops *c, const void *ptr, int size, int maxnum);
static int rw_sock_close(SDL_RWops *c);

SDL_RWops *SDL_RWFromSock(TCPsocket sock, int autoclose)
{
  SDL_RWops *rw=SDL_AllocRW();
  if(rw==NULL) return(NULL);

  rw->seek =rw_sock_seek;
  rw->read =rw_sock_read;
  rw->write=rw_sock_write;
  rw->close=rw_sock_close;
  rw->type =SDL_SOCK_TYPE;
  rw->hidden.stdio.fp=(FILE *)sock;
  rw->hidden.stdio.autoclose=1;
  return(rw);
}

SDL_RWops *SDL_RWFromHost(const char *host, Uint16 port)
{
  SDL_RWops *rw=SDL_AllocRW();
  IPaddress ip;
  TCPsocket sock;

  if(rw==NULL)
    return(NULL);

  if(SDLNet_ResolveHost(&ip,host,port)<0)
  {
    SDL_FreeRW(rw);
    return(NULL);
  }

  sock=SDLNet_TCP_Open(&ip);
  if(sock==NULL)
  {
    SDL_FreeRW(rw);
    return(NULL);  
  }

  rw->seek =rw_sock_seek;
  rw->read =rw_sock_read;
  rw->write=rw_sock_write;
  rw->close=rw_sock_close;
  rw->type=SDL_SOCK_TYPE;
  rw->hidden.stdio.fp=(FILE *)sock;
  rw->hidden.stdio.autoclose=1;
  return(rw);
}

static int rw_sock_seek(SDL_RWops *c, int offset, int whence)
{
  SDL_SetError("rw_sock_seek:  Seeking unimplimented on sockets");
  return(-1);
}

static int rw_sock_read (SDL_RWops *c, void *ptr, int size, int maxnum)
{
  return(SDLNet_TCP_Recv((TCPsocket)(c->hidden.stdio.fp),ptr,size*maxnum));
}

static int rw_sock_write(SDL_RWops *c, const void *ptr, int size, int maxnum)
{
  
return(SDLNet_TCP_Send((TCPsocket)(c->hidden.stdio.fp),(void *)ptr,size*maxnum));
}

static int rw_sock_close(SDL_RWops *c)
{
  if(c==NULL)
  {
    SDL_SetError("rw_sock_close:  Invalid RWops");
    return(-1);
  }
  else if(c->type!=SDL_SOCK_TYPE)
  {
    SDL_SetError("rw_sock_close:  RWops is the wrong type");
    return(-1);
  }

  if(c->hidden.stdio.autoclose)
  {
    SDLNet_TCP_Close((TCPsocket)(c->hidden.stdio.fp));
  }
  SDL_FreeRW(c);
  return(0);
}

#ifdef RW_SOCK_TEST

#include <SDL/SDL_thread.h>
#include "SDL_rwfd.h"

int running=2;

int threadread(SDL_RWops *sock)
{
  while(running)
  {
    char buf[512];
    int size=SDL_RWread(sock,buf,1,512);
    if(size>0) SDL_RWwrite(rwout,buf,1,size);
    else break;
  }

  rwprintf(rwerr,"Read thread exiting\n");
  running--;
  return(1);
}

int threadwrite(SDL_RWops *sock)
{
  while(running>1)
  {
    char buf[512];
    int size=SDL_RWread(rwin,buf,1,512);
    if(size>0) SDL_RWwrite(sock,buf,1,size);
    else if(size<=0) break;
  }

  rwprintf(rwerr,"Write thread exiting\n");
  running--;
  return(1);
}

int main(int argc, char *argv[])
{
  SDL_RWops *s;
  SDL_Init(0);
  SDLNet_Init();

  s=SDL_RWFromHost("smtmuck.dyndns.org",8888);
  if(s==NULL)
  {
    rwprintf(rwerr,"Couldn't open socket\n");
    SDLNet_Quit();
    SDL_Quit();
    return(1);
  }

  SDL_CreateThread(threadwrite,s);
  SDL_CreateThread(threadread,s);

  while(running)
    SDL_Delay(100);

  SDLNet_Quit();
  SDL_Quit();
  return(0);
}
#endif/*RW_SOCK_TEST*/
