/***************************************************************************
 *   SDL_rwz.c -- SDL_RWops zlib compression/decompression stream          *
 *     implimentation                                                      *
 *                                                                         *
 *   Copyright (C) 2004 by Tyler Montbriand                                *
 *   tsm@accesscomm.ca                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include <zlib.h>
#include <SDL/SDL_thread.h>
#include "SDL_rwz.h"

#define SDL_RW_GZ 0xdead0003

static int rw_gz_seek (SDL_RWops *c, int offset, int whence);
static int rw_gz_read (SDL_RWops *c, void *ptr, int size, int maxnum);
static int rw_gz_write(SDL_RWops *c, const void *ptr, int size, int maxnum);
static int rw_gz_close(SDL_RWops *c);

SDL_RWops *SDL_RWFromGZ(const char *file, const char *mode)
{
  SDL_RWops *rw=SDL_AllocRW();
  gzFile f;

  if(rw==NULL) return(NULL);

  f=gzopen(file,mode);
  if(f==NULL)
  {
    SDL_SetError("Could not open/create gz");
    SDL_FreeRW(rw);
    return(NULL);
  }
  rw->seek =rw_gz_seek;
  rw->read =rw_gz_read;
  rw->write=rw_gz_write;
  rw->close=rw_gz_close;
  rw->type =SDL_RW_GZ;
  rw->hidden.unknown.data1=f;
  return(rw);
}

static int rw_gz_seek (SDL_RWops *c, int offset, int whence)
{
  return(gzseek((gzFile)(c->hidden.unknown.data1),offset,whence));
}

static int rw_gz_read (SDL_RWops *c, void *ptr, int size, int maxnum)
{
  int bytes=gzread((gzFile)(c->hidden.unknown.data1),ptr,size*maxnum);

  if(bytes%size)
    gzseek((gzFile)(c->hidden.unknown.data1),-(bytes%size),SEEK_CUR);

  return(bytes/size);
}

static int rw_gz_write(SDL_RWops *c, const void *ptr, int size, int maxnum)
{
  int bytes=gzwrite((gzFile)(c->hidden.unknown.data1),ptr,size*maxnum);

  if(bytes%size)
    gzseek((gzFile)(c->hidden.unknown.data1),-(bytes%size),SEEK_CUR);

  return(bytes/size);
}

static int rw_gz_close(SDL_RWops *c)
{
  if(c->type!=SDL_RW_GZ)
  {
    SDL_SetError("rw_gz_close:  SDL_RWops is the wrong type!");
    return(-1);
  }

  gzclose((gzFile)(c->hidden.unknown.data1));
  c->hidden.unknown.data1=NULL;
  SDL_FreeRW(c);
  return(0);
}
